package com.yardpros;
import java.util.*;

public class User extends LinkedHashMap
{
  public User()
  {
    this.put("username", ""); 
    this.put("password", "");
    this.put("!password2", "");
    this.put("firstname", "");
    this.put("0middlename", "");
    this.put("lastname", "");
    this.put("address", "");
    this.put("0address2", "");
    this.put("city", "");
    this.put("state", "");
    this.put("zipcode", "");
    this.put("country", "");
    this.put("email", "");
    this.put("0phone", "");
    this.put("0admin", "");
    this.put("0notes", "");
  }
  /**
   * This will construct a predefined fields based on a java.util.Map. 
   * pre: All fields that are not required must be prefixed by '0'
          All fields (such as a password check) must be prefixed by '!'
   * 'username' and 'password' must correspond to their respective keys.
   */
  public User(Map<String,String> m)
  {
    this.putAll(m);
  }
  /**
   * Because reference equality is not handled well with session scope java
   * beans, use this method to set all of the parameters of different instance
   * of User to this user.
   * @param (User)anotherUser - the user to be made into this user.
   */
   
   public void copyInto(User anotherUser)
   {
     this.putAll(anotherUser);
   }
  
   /**
    * This wraps all of the conditions for the required this.
    * It tests: 1. All required fields are filled in.
    *           2. The passwords match
    *           3. An email has the appropriate characters - '@' and '.'
    * @throws InvalidInputException - if the input is invalid
    */
   public void checkFields() throws InvalidInputException
   {
     checkEmptyFields();
     checkPassword();
     checkEmail();
   }
   
   /**
    * This method checks that all the required fields are filled in.
    * Fields that are not required should be prefixed with the char '0'.
    * pre: the keys in fields must be initialized
    * post: the values in fields are valid for an SQL query
    * @throws InvalidInputException - if the field is empty
    */
   public void checkEmptyFields() throws InvalidInputException
   {
     for(String i: (String[])this.keySet().toArray(new String[0]))
       if(!(i.startsWith("0")) && (this.get(i).equals("") || this.get(i).equals(null)))
           throw new InvalidInputException("You must enter a valid "+i);
   }
     
   /**
    * Checks to see that passwords match in both password this.
    * @throws InvalidInputException() - if the password fields do not match
    */
   public void checkPassword() throws InvalidInputException
   {
     if( !( get("password").equals(get("!password2")) ) )
       throw new InvalidInputException("Your passwords did not match");
   }
     
   /**
    * Checks to see if the email syntax is valid
    * @throws InvalidInputException - if the syntax is incorrect
    */
   public void checkEmail() throws InvalidInputException
   {
     String em = (String)get("email");
     if( em.indexOf("@") < 0 ||
       em.lastIndexOf(".") < em.indexOf("@") )
        throw new InvalidInputException("Your email is not valid");
   }
   
   /**
    *Clears all of the attributes of the user
    */
    public void clear()
    {
      for(Object i:keySet().toArray())
        put(i,"");
    }
    
   
  /*
	 * Returns the value of username.
	 *
	public String getUsername() 
	{
		return username;
	}

	/**
	 * Sets the value of username.
	 * @param username The value to assign username.
   * @throws InvalidInputException
	 *
	public void setUsername(String username) 
                          throws com.yardpros.InvalidInputException
	{
    this.username = username;
    if( username.equals("")|| username.equals(null))
      throw new com.yardpros.InvalidInputException("You must enter a username");
	}

	/**
	 * Returns the value of password.
	 *
	public String getPassword()
	{
		return password;
	}

	/**
	 * Sets the value of password from two password input this. If the passwords
   * don't match then it returns a false.
	 * @param password1 The first input to assign password.
   * @param password2 The second input tp assign password.
   * @throws InvalidInputException - if the two passowords don't match.
   * @throws InvalidInputException - if the field is blank
	 *
	public void setPassword(String password1, String password2)
                  throws com.yardpros.InvalidInputException
	{
    if( password1.equals("")|| password1.equals(null))
      throw new com.yardpros.InvalidInputException("You must enter a password");
    
    password = password1.equals(password2) ? password1 : "mismatch";
    
    if(password.equals("mismatch"))
      throw new com.yardpros.InvalidInputException("the passwords did not match");
	}

	/**
	 * Returns the value of firstname.
	 *
	public String getFirstname()
	{
		return firstname;
	}

	/**
	 * Sets the value of firstname.
	 * @param firstname The value to assign firstname.
   * @throws InvalidInputException
	 *
	public void setFirstname(String firstname) throws com.yardpros.InvalidInputException
	{
    
    if( firstname.equals("")|| firstname.equals(null))
      throw new com.yardpros.InvalidInputException("You must enter a first name");
    this.firstname = firstname;
	}
	/**
	 * Returns the value of lastname.
	 *
	public String getLastname()
	{
		return lastname;
	}
  
  /**
	 * Returns the value of middlename.
	 *
	public String getMiddlename()
	{
		return middlename;
	}

	/**
	 * Sets the value of middlename.
	 * @param middlename The value to assign middlename.
	 *
	public void setMiddlename(String middlename) 
	{
    if( middlename.equals("")|| middlename.equals(null))
      this.middlename = "NULL";
    this.middlename = middlename;
	}

  
	/**
	 * Sets the value of lastname.
	 * @param lastname The value to assign lastname.
   * @throws InvalidInputException
	 *
	public void setLastname(String lastname)
                          throws com.yardpros.InvalidInputException
	{
    if( lastname.equals("")|| lastname.equals(null))
      throw new com.yardpros.InvalidInputException("You must enter a valid last name");
    this.lastname = lastname;
	}

	/**
	 * Returns the value of address.
	 *
	public String getAddress()
	{
		return address;
	}

	/**
	 * Sets the value of address.
	 * @param address The value to assign address.
   * @throws InvalidInputException
	 *
	public void setAddress(String address)
                          throws com.yardpros.InvalidInputException
	{
    if( address.equals("")|| address.equals(null))
      throw new com.yardpros.InvalidInputException("You must enter a valid address");
    this.address = address;
	}

	/**
	 * Returns the value of address2.
	 *
	public String getAddress2()
	{
		return address2;
	}

	/**
	 * Sets the value of address2.
	 * @param address2 The value to assign address2.
	 *
	public void setAddress2(String address2)
	{
    if( address2.equals("")|| address2.equals(null))
      this.address2 = "NULL";
    this.address2 = address2;
	}

	/**
	 * Returns the value of city.
	 *
	public String getCity()
	{
		return city;
	}

	/**
	 * Sets the value of city.
	 * @param city The value to assign city.
   * @throws InvalidInputException
	 *
	public void setCity(String city)
                          throws com.yardpros.InvalidInputException
	{
    if( city.equals("")|| city.equals(null))
      throw new com.yardpros.InvalidInputException("You must enter a valid city");
    this.city = city;
	}

	/**
	 * Returns the value of state.
	 *
	public String getState()
	{
		return state;
	}

	/**
	 * Sets the value of state.
	 * @param state The value to assign state.
   * @throws InvalidInputException
	 *
	public void setState(String state)
                          throws com.yardpros.InvalidInputException
	{
    if( state.equals("") || state.equals(null) )
      throw new com.yardpros.InvalidInputException("You must enter a valid state");
    this.state = state;
	}

	/**
	 * Returns the value of zipcode.
	 *
	public String getZipcode()
	{
		return zipcode;
	}

	/**
	 * Sets the value of zipcode.
	 * @param zipcode The value to assign zipcode.
   * @throws InvalidInputException
	 *
	public void setZipcode(String zipcode)
                          throws com.yardpros.InvalidInputException
	{
    if( zipcode.equals("")|| zipcode.equals(null))
      throw new com.yardpros.InvalidInputException("You must enter a valid zip code");
    this.zipcode = zipcode;
	}

	/**
	 * Returns the value of country.
	 *
	public String getCountry()
	{
		return country;
	}

	/**
	 * Sets the value of country.
	 * @param country The value to assign country.
   * @throws InvalidInputException
	 *
	public void setCountry(String country)
                          throws com.yardpros.InvalidInputException
	{
    if( country.equals("")|| country.equals(null))
      throw new com.yardpros.InvalidInputException("You must enter a valid country");
    this.country = country;
	}

	/**
	 * Returns the value of email.
	 *
	public String getEmail()
	{
		return email;
	}

	/**
	 * Sets the value of email.
	 * @param email The value to assign email.
   * @throws InvalidInputException
	 *
	public void setEmail(String email)
                          throws com.yardpros.InvalidInputException
	{
    if( email.equals("")|| email.equals(null))
      throw new com.yardpros.InvalidInputException("You must enter a valid email");
    this.email = email;
	}

	/**
	 * Returns the value of phone.
	 *
	public String getPhone()
	{
		return phone;
	}

	/**
	 * Sets the value of phone.
	 * @param phone The value to assign phone.
	 *
	public void setPhone(String phone)
	{
    if( phone.equals("")|| phone.equals(null))
      this.phone = "NULL";
    this.phone = phone;
	}

	/**
	 * Returns the value of admin.
	 *
	public boolean getAdmin()
	{
		return admin;
	}

	/**
	 * Sets the value of admin.
	 * @param admin The value to assign admin.
	 *
	public void setAdmin(boolean admin)
	{
		this.admin = admin;
	}

	/**
	 * Returns the value of notes.
	 *
	public String getNotes()
	{
		return notes;
	}

	/**
	 * Sets the value of notes.
	 * @param notes The value to assign notes.
	 *
	public void setNotes(String notes)
	{
    if( notes.equals("")|| notes.equals(null))
      this.notes = "NULL";
    this.notes = notes;
	}
  *
 
  /**
   * Gets all of the user information and returns as an array string
   * @return String[] - All of the user information.
   *
   public String[] getAll()
   {
     String [] all = new String [NUMFIELDS];
     
     all[0] = getUsername();
     all[1] = getPassword();
     all[2] = getFirstname();
     all[3] = getMiddlename();
     all[4] = getLastname();
     all[5] = getAddress();
     all[6] = getAddress2();
     all[7] = getCity();
     all[8] = getState();
     all[9] = getZipcode();
     all[10] = getCountry();
     all[11] = getEmail();
     all[12] = getPhone();
     all[13] = getNotes();
     all[14] = Boolean.toString(getAdmin());
     
     return all;
   }*/
   
  
}
